@@ -25,6 +25,9 @@ DEFAULT_START_ENV = (
     "MWWM=allwm",
     "#force GTK3 applications to use X11 so we can intercept them:",
     "GDK_BACKEND=x11",
+    "#tell GTK4 applications to use the software Gsk renderer to avoid freezes and crashes:",
+    "#(if using a real GPU, you may want to switch to `vulkan` or `opengl` instead)",
+    "GSK_RENDERER=cairo",
     "#force Qt applications to use X11 so we can intercept them:",
     "QT_QPA_PLATFORM=xcb",
     "#disable Qt scaling:"
@@ -34,6 +37,8 @@ DEFAULT_START_ENV = (
     "GTK_OVERLAY_SCROLLING=0",
     "#some versions of GTK3 honour this option, sadly not all:",
     "GTK_CSD=0",
+    "#use this to make SDL video windows minimize on focus loss:",
+    "#SDL_VIDEO_MINIMIZE_ON_FOCUS_LOSS=1",
 )
 
 DEFAULT_SSH_CMD = "ssh"
