@@ -8,20 +8,17 @@ from typing import Any
 
 from xpra.os_util import gi_import
 from xpra.scripts.config import InitException
-from xpra.net.common import PacketType
+from xpra.net.common import Packet
 from xpra.x11.desktop.base import DesktopServerBase
-from xpra.x11.desktop.monitor_model import MonitorDesktopModel
-from xpra.server.mixins.window import WindowsMixin
-from xpra.x11.vfb_util import parse_resolution
+from xpra.server.subsystem.window import WindowsConnection
+from xpra.x11.common import X11Event
 from xpra.x11.bindings.randr import RandRBindings
-from xpra.gtk.error import xsync, xlog
+from xpra.x11.error import xsync, xlog, XError
 from xpra.log import Logger
 
 GObject = gi_import("GObject")
 GLib = gi_import("GLib")
 
-RandR = RandRBindings()
-
 log = Logger("server")
 metadatalog = Logger("x11", "metadata")
 screenlog = Logger("screen")
@@ -33,7 +30,34 @@ MAX_SIZE = 8192, 8192
 
 def get_screen_size() -> tuple[int, int]:
     with xsync:
-        return RandR.get_screen_size()
+        return RandRBindings().get_screen_size()
+
+
+def apply_monitor_config(monitor_defs: dict) -> None:
+    try:
+        with xsync:
+            RandRBindings().set_crtc_config(monitor_defs)
+    except XError as e:
+        screenlog("apply_monitor_config(%s)", monitor_defs, exc_info=True)
+        screenlog.error("Error applying changes to the monitor configuration")
+        screenlog.estr(e)
+
+
+def _adjust_monitor(model, delta_x: int, delta_y: int) -> None:
+    screenlog("adjust_monitors(%s, %i, %i)", model, delta_x, delta_y)
+    if delta_x == 0 and delta_y == 0:
+        return
+    x, y = model.get_geometry()[:2]
+    new_x = max(0, x + delta_x)
+    new_y = max(0, y + delta_y)
+    if new_x != x or new_y != y:
+        screenlog(f"adjusting monitor {model} from {x},{y} to {new_x},{new_y}")
+        mdef = model.get_definition()
+        mdef |= {
+            "x": new_x,
+            "y": new_y,
+        }
+        model.init(mdef)
 
 
 class XpraMonitorServer(DesktopServerBase):
@@ -43,28 +67,28 @@ class XpraMonitorServer(DesktopServerBase):
     __gsignals__ = DesktopServerBase.__common_gsignals__
 
     def __init__(self):
-        with xsync:
-            if not RandR.is_dummy16():
-                display = os.environ.get("DISPLAY", "")
-                raise InitException(f"the vfb display {display!r} cannot virtualize monitors - dummy RandR 1.6 missing")
         super().__init__()
-        self.session_type: str = "monitor"
+        self.session_type: str = "X11 monitor"
         self.reconfigure_timer: int = 0
         self.reconfigure_locked: bool = False
 
+    def setup(self):
+        super().setup()
+        with xsync:
+            if not RandRBindings().is_dummy16():
+                display = os.environ.get("DISPLAY", "")
+                raise InitException(f"the vfb display {display!r} cannot virtualize monitors - dummy RandR 1.6 missing")
+
     def init_randr(self) -> None:
         super().init_randr()
         from xpra.x11.vfb_util import set_initial_resolution
         screenlog(f"init_randr() randr={self.randr}, initial-resolutions={self.initial_resolutions}")
-        if not RandR.has_randr() or not self.initial_resolutions:
+        if not RandRBindings().has_randr() or not self.initial_resolutions:
             return
         res = self.initial_resolutions
         with xlog:
             set_initial_resolution(res, self.dpi or self.default_dpi)
 
-    def get_server_mode(self) -> str:
-        return "X11 monitor"
-
     def make_hello(self, source) -> dict[str, Any]:
         capabilities = super().make_hello(source)
         if "features" in source.wants:
@@ -77,7 +101,7 @@ class XpraMonitorServer(DesktopServerBase):
             }
         return capabilities
 
-    def configure_best_screen_size(self):
+    def configure_best_screen_size(self) -> tuple[int, int]:
         sss = tuple(x for x in self._server_sources.values() if x.ui_client)
         log(f"configure_best_screen_size() sources={sss}")
         if len(sss) != 1:
@@ -99,23 +123,29 @@ class XpraMonitorServer(DesktopServerBase):
             log("Warning: failed to mirror client monitor layout", exc_info=True)
             self.reconfigure_locked = False
 
-        def unlock():
+        def unlock() -> None:
             self.reconfigure_locked = False
 
         GLib.timeout_add(1000, unlock)
         return get_screen_size()
 
     def load_existing_windows(self) -> None:
-        with xlog:
-            monitors = RandR.get_monitor_properties()
+        with (xlog):
+            monitors = RandRBindings().get_monitor_properties()
+            screenlog("load_existing_windows() found monitors=%r", monitors)
+            outputs = set()
             for i, monitor in monitors.items():
-                self.add_monitor_model(i + 1, monitor)
+                moutputs = set(monitor.get("outputs", ()))
+                # skip this monitor if we already have another one with the same outputs:
+                if monitor.get("primary") or not outputs.issuperset(moutputs):
+                    outputs.update(moutputs)
+                    self.add_monitor_model(i + 1, monitor)
         # does not fire: (because of GTK?)
         # RandR.select_crtc_output_changes()
         # screen = gdk.Screen.get_default()
         # screen.connect("monitors-changed", self.monitors_changed)
 
-    def do_x11_configure_event(self, event) -> None:
+    def do_x11_configure_event(self, event: X11Event) -> None:
         # the root window changed,
         # check to see if a monitor has been modified
         # do this via a timer to avoid running multiple times
@@ -136,7 +166,7 @@ class XpraMonitorServer(DesktopServerBase):
         #  ie: `xrandr --output DUMMY1 --mode 1024x768`
         mdefs = {}
         with xlog:
-            info = RandR.get_all_screen_properties()
+            info = RandRBindings().get_all_screen_properties()
             crtcs = info.get("crtcs")
             outputs = info.get("outputs")
             monitors = info.get("monitors")
@@ -170,7 +200,7 @@ class XpraMonitorServer(DesktopServerBase):
                     screenlog(f"no monitor found for output id {output_id}")
                     return
                 # get the geometry from the crtc:
-                mdef = {k: v for k, v in crtc_info.items() if k in ("x", "y", "width", "height")}
+                mdef: dict[str, int] = {k: v for k, v in crtc_info.items() if k in ("x", "y", "width", "height")}
                 # add the millimeter dimensions from the output:
                 mdef.update((k, v) for k, v in output_info.items() if k in ("mm-width", "mm-height"))
                 # and some monitor attributes:
@@ -210,7 +240,8 @@ class XpraMonitorServer(DesktopServerBase):
                 model.notify("title")
         return mods
 
-    def add_monitor_model(self, wid: int, monitor) -> MonitorDesktopModel:
+    def add_monitor_model(self, wid: int, monitor):
+        from xpra.x11.desktop.monitor_model import MonitorDesktopModel
         model = MonitorDesktopModel(monitor)
         model.setup()
         screenlog("adding monitor model %s", model)
@@ -224,7 +255,7 @@ class XpraMonitorServer(DesktopServerBase):
         delta_x, delta_y = model.resize_delta
         wid = self._window_to_id[model]
         w, h = model.get_dimensions()
-        screenlog("monitor_resized(%s) size=%s, delta=%s, wid=%i",
+        screenlog("monitor_resized(%s) size=%s, delta=%s, wid=%#x",
                   model, (w, h), (delta_x, delta_y), wid)
         for ss in self.window_sources():
             ss.resize_window(wid, model, w, h)
@@ -241,7 +272,7 @@ class XpraMonitorServer(DesktopServerBase):
         # now we can do the virtual crtcs, outputs and monitors
         defs = self.get_monitor_config()
         screenlog("reconfigure_monitors() definitions=%s", defs)
-        self.apply_monitor_config(defs)
+        apply_monitor_config(defs)
         # and tell the client:
         self.setting_changed("monitors", defs)
 
@@ -263,23 +294,7 @@ class XpraMonitorServer(DesktopServerBase):
         if (delta_x == 0 and delta_y == 0) or not models:
             return
         for wid, model in models.items():
-            self._adjust_monitor(model, delta_x, delta_y)
-
-    def _adjust_monitor(self, model, delta_x: int, delta_y: int) -> None:
-        screenlog("adjust_monitors(%s, %i, %i)", model, delta_x, delta_y)
-        if delta_x == 0 and delta_y == 0:
-            return
-        x, y = model.get_geometry()[:2]
-        new_x = max(0, x + delta_x)
-        new_y = max(0, y + delta_y)
-        if new_x != x or new_y != y:
-            screenlog(f"adjusting monitor {model} from {x},{y} to {new_x},{new_y}")
-            mdef = model.get_definition()
-            mdef |= {
-                "x": new_x,
-                "y": new_y,
-            }
-            model.init(mdef)
+            _adjust_monitor(model, delta_x, delta_y)
 
     def get_monitor_config(self) -> dict[int, dict]:
         monitor_defs = {}
@@ -290,15 +305,11 @@ class XpraMonitorServer(DesktopServerBase):
             monitor_defs[i] = monitor
         return monitor_defs
 
-    def apply_monitor_config(self, monitor_defs: dict) -> None:
-        with xsync:
-            RandR.set_crtc_config(monitor_defs)
-
     def remove_monitor(self, wid: int) -> None:
         model = self._id_to_window.get(wid)
         screenlog("removing monitor for wid %i : %s", wid, model)
         if not model:
-            raise ValueError(f"monitor {wid} not found")
+            raise ValueError(f"monitor {wid:#x} not found")
         if len(self._id_to_window) <= 1:
             raise RuntimeError("cannot remove the last monitor")
         delta_x = -model.get_definition().get("width", 0)
@@ -323,29 +334,21 @@ class XpraMonitorServer(DesktopServerBase):
         # find the wid to use:
         # prefer just incrementing the wid, but we cannot go higher than 16
 
-        def rightof(wid: int):
-            mdef = self._id_to_window[wid].get_definition()
-            x = mdef.get("x", 0) + mdef.get("width", 0)
-            y = mdef.get("y", 0)  # +monitor.get("height", 0)
-            return x, y
+        def rightof(other_wid: int):
+            mdef = self._id_to_window[other_wid].get_definition()
+            ox = mdef.get("x", 0) + mdef.get("width", 0)
+            oy = mdef.get("y", 0)  # +monitor.get("height", 0)
+            return ox, oy
 
-        wid = self._max_window_id
+        # find a gap we can use in the window ids before 16:
+        wid = 1
+        for wid in range(1, 16):
+            if wid not in self._id_to_window:
+                break
+        screenlog(f"using {wid=}")
         x = y = 0
-        if wid < 16:
-            # since we're just appending,
-            # just place to the right of the last monitor:
-            last = max(self._id_to_window)
-            x, y = rightof(last)
-        else:
-            # find a gap we can use in the window ids before 16:
-            prev = None
-            for wid in range(16):
-                if wid not in self._id_to_window:
-                    break
-                prev = wid
-            assert wid <= 16
-            if prev:
-                x, y = rightof(prev)
+        if wid > 1:
+            x, y = rightof(wid - 1)
             self._adjust_monitors(wid - 1, width, 0)
         # ensure no monitors end up too far to the right or bottom:
         # (better have them overlap - though we could do something smarter here)
@@ -371,26 +374,26 @@ class XpraMonitorServer(DesktopServerBase):
         self.reconfigure_monitors()
         # send it to the clients:
         for ss in self._server_sources.values():
-            if not isinstance(ss, WindowsMixin):
+            if not isinstance(ss, WindowsConnection):
                 continue
             self.send_new_desktop_model(model, ss)
 
-    def _process_configure_monitor(self, _proto, packet: PacketType) -> None:
-        action = str(packet[1])
+    def _process_configure_monitor(self, _proto, packet: Packet) -> None:
+        action = packet.get_str(1)
         if action == "remove":
-            identifier = str(packet[2])
-            value = packet[3]
+            identifier = packet.get_str(2)
             if identifier == "wid":
-                wid = int(value)
+                wid = packet.get_wid(3)
             elif identifier == "index":
                 # index is zero-based
-                wid = int(value) + 1
+                wid = packet.get_u32(3) + 1
             else:
                 raise ValueError(f"unsupported monitor identifier {identifier!r}")
             self.remove_monitor(wid)
         elif action == "add":
             resolution = packet[2]
             if isinstance(resolution, str):
+                from xpra.common import parse_resolution
                 resolution = parse_resolution(resolution, self.refresh_rate)
             if not isinstance(resolution, (tuple, list)):
                 raise ValueError(f"invalid resolution: {resolution!r} ({type(resolution)}")
